#include "fintek_api.h"


void usage(int argc, char* argv[])
{
	fprintf(stderr, "\n\n\n\nFintek GPIO Demo API:\n");
	fprintf(stderr, "%s <idx> <direct> <type> <value>\n\n" , argv[0]);
	fprintf(stderr, "Param List:\n");
	fprintf(stderr, "\tidx: number of GPIO, lke GPIO17, idx=0x17\n");
	fprintf(stderr, "\tdir: 0: In, 1: Out\n");
	fprintf(stderr, "\ttype: 0: Open Drain, 1: Push-Pull\n");
	fprintf(stderr, "\tvalue: 0/1\n");
}


int gpio_demo(int argc, char* argv[])
{
	unsigned int idx = 0;
	unsigned int data = 0;

	if(argc < 5) {
		usage(argc, argv);
		exit(3);
	}

	idx = strtol(argv[1], NULL, 16);
	CHECK_RET(_EnableGPIO(idx , eGPIO_Mode_Enable));
	printf("idx: 0x%x => set:%x, count:%x\n",
		idx, (idx >> 4) & 0xf, (idx >> 0) & 0xf);

	// out
	data = strtol(argv[2], NULL, 16);
	CHECK_RET(_SetGpioOutputEnableIdx(idx , data));
	printf("Dir: 0x%x => %s\n", data,
		(data == eGPIO_Direction_Out) ? "OUT" : "IN");

	if(data == eGPIO_Direction_Out) {

		data = strtol(argv[3], NULL, 16);
		CHECK_RET(_SetGpioDriveEnable(idx , data));
		printf("Mode: 0x%x => %s\n", data,
			(data == eGPIO_Drive_Mode_OpenDrain)
				? "OpenDrain" : "Push-Pull");

		data = strtol(argv[4], NULL, 16);
		CHECK_RET(_SetGpioOutputDataIdx(idx , data));

		printf("Write Value: 0x%x \n", data);
	} else {

		CHECK_RET(_GetGpioInputDataIdx(idx , &data));
		printf("Read Value: 0x%x \n", data);
	}

	return 0;
}


int main(int argc, char* argv[])
{
	int nRet = 0;
	sFintek_sio_data sio_data;

	/* Please set this to 1 enable more debug message */
	set_debug(1);

	fprintf(stderr, "FINTEK Lib Version: %s\n", getFintekLibVersion());

	if( nRet = init_fintek_sio(eSIO_TYPE_F81866, 0 ,&sio_data) ) {
		fprintf(stderr, "init_fintek_sio error\n");
		exit(3);
	}

	ActiveSIO(sio_data.ic_port, sio_data.key);
	gpio_demo(argc, argv);
	DeactiveSIO(sio_data.ic_port);


	return 0;
}

