#include "fintek_api.h"

static int hwmon_io_addr = 0;

static unsigned char READ_HWMON(unsigned long reg)
{
	unsigned char data = 0;
	IOWriteByte(hwmon_io_addr, reg);
	data = IOReadByte(hwmon_io_addr + 1);
	return data;
}

static void WRITE_HWMON(unsigned long reg, unsigned char data)
{
	IOWriteByte(hwmon_io_addr, reg);
	IOWriteByte(hwmon_io_addr + 1, data);
}

void usage(char *argv[])
{
	printf("Fintek I2C Tools:");
	printf("Usage:\n");
	printf("%s [r,w] [channel] [addr] [data]\n", argv[0]);
	printf("all value are hex format\n");
}

int i2c_demo(int argc, char *argv[])
{
#define USAGE_API

	int status;
	unsigned int addr = 0;
	unsigned int tmp = 0;

#ifdef USAGE_API
	unsigned int channel = 0;

	if (argc < 2) {
		usage(argv);
		return -1;
	}

	if (strlen(argv[1]) != 1) {
		usage(argv);
		return -1;
	}

	if ((strcmp(argv[1], "r") == 0) && (argc >= 4)) {
		channel = strtol(argv[2], NULL, 16);
		addr = strtol(argv[3], NULL, 16);

		printf("Read channel:%x addr:%x\n", channel, addr);
		/*
		 * F81866 I2C 
		 *      idx 0: for PIN 67/68
		 *      idx 1: for PIN 71/76
		 *      idx 2: for PIN 61/62
		 */
		status = SelectI2CChannel(0);
		if (status) {
			fprintf(stderr, "%s: SelectI2CChannel error: %d\n",
				__func__, status);
			return status;
		}

		/* Read I2C address 0x20 (dont care bit0) */
		status = ReadI2CData(addr, &tmp);
		if (status) {
			fprintf(stderr, "%s: ReadI2CData error: %d\n",
				__func__, status);
			return status;
		}

		printf("Read value:%x\n", tmp);
	} else if ((strcmp(argv[1], "w") == 0) && (argc >= 5)) {
		channel = strtol(argv[2], NULL, 16);
		addr = strtol(argv[3], NULL, 16);
		tmp = strtol(argv[4], NULL, 16);

		printf("Write channel:%x addr:%x data:%x\n", channel, addr,
		       tmp);
		/*
		 * F81866 I2C 
		 *      idx 0: for PIN 67/68
		 *      idx 1: for PIN 71/76
		 *      idx 2: for PIN 61/62
		 */
		status = SelectI2CChannel(0);
		if (status) {
			fprintf(stderr, "%s: SelectI2CChannel error: %d\n",
				__func__, status);
			return status;
		}

		/* Read I2C address 0x20 (dont care bit0) */
		status = WriteI2CData(addr, tmp);
		if (status) {
			fprintf(stderr, "%s: WriteI2CData error: %d\n",
				__func__, status);
			return status;
		}

	} else {
		usage(argv);
		return -1;
	}

	return 0;
#else
	/* Get HWMON Base IO Address */
	status = GetHWMONAddr(&hwmon_io_addr);
	if (status) {
		fprintf(stderr, "%s: InitI2C error: %d\n", __func__, status);
		return status;
	}

	printf("%s: get hwmon addr %04x\n", __func__, hwmon_io_addr);

	/* Examples: Read I2C status */
	tmp = READ_HWMON(0xEE);
	printf("%s: read I2C status:%02x\n", __func__, tmp);

	/* Examples: I2C start with Read Byte */
	WRITE_HWMON(0xEF, 0x8A);
#endif

	return 0;
}

int main(int argc, char *argv[])
{
	int nRet = 0;
	sFintek_sio_data sio_data;

	fprintf(stderr, "FINTEK Lib Version: %s\n", getFintekLibVersion());

	if (nRet = init_fintek_sio(eSIO_TYPE_F81866, 0, &sio_data)) {
		fprintf(stderr, "init_fintek_sio error\n");
		exit(3);
	}

	ActiveSIO(sio_data.ic_port, sio_data.key);
	i2c_demo(argc, argv);
	DeactiveSIO(sio_data.ic_port);

	return 0;
}
